@@ -1,12 +1,38 @@
-NUM_VERSION = (2015, 4, 1)
-VERSION = ".".join(str(nv) for nv in NUM_VERSION)
-__version__ = VERSION
+from __future__ import absolute_import, division, print_function
+
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
 
 from pudb.py3compat import raw_input, PY3
+from pudb.settings import load_config
 
-from pudb.settings import load_config, save_config
-CONFIG = load_config()
-save_config(CONFIG)
+
+NUM_VERSION = (2020, 1)
+VERSION = ".".join(str(nv) for nv in NUM_VERSION)
+__version__ = VERSION
 
 
 class PudbShortcuts(object):
@@ -15,9 +41,22 @@ class PudbShortcuts(object):
         import sys
         dbg = _get_debugger()
 
-        set_interrupt_handler()
+        import threading
+        if isinstance(threading.current_thread(), threading._MainThread):
+            set_interrupt_handler()
         dbg.set_trace(sys._getframe().f_back)
 
+    @property
+    def go(self):
+        import sys
+        dbg = _get_debugger()
+
+        import threading
+        if isinstance(threading.current_thread(), threading._MainThread):
+            set_interrupt_handler()
+        dbg.set_trace(sys._getframe().f_back, paused=False)
+
+
 if PY3:
     import builtins
     builtins.__dict__["pu"] = PudbShortcuts()
@@ -29,8 +68,42 @@ else:
 CURRENT_DEBUGGER = []
 
 
+def _tty_override():
+    import os
+    return os.environ.get("PUDB_TTY")
+
+
+def _open_tty(tty_path):
+    import io
+    import os
+    import sys
+    if sys.version_info[0] == 2:
+        import fcntl
+        import termios
+        import struct
+        tty_file = open(tty_path, "r+b", buffering=0)
+        try:
+            s = struct.unpack("hh", fcntl.ioctl(
+                tty_file.fileno(), termios.TIOCGWINSZ, "1234"))
+            term_size = (s[1], s[0])
+        except Exception:
+            term_size = None
+    else:
+        tty_file = io.TextIOWrapper(open(tty_path, "r+b", buffering=0))
+        term_size = os.get_terminal_size(tty_file.fileno())
+
+    return tty_file, term_size
+
+
 def _get_debugger(**kwargs):
     if not CURRENT_DEBUGGER:
+        tty_path = _tty_override()
+        if tty_path and ("stdin" not in kwargs or "stdout" not in kwargs):
+            tty_file, term_size = _open_tty(tty_path)
+            kwargs.setdefault("stdin", tty_file)
+            kwargs.setdefault("stdout", tty_file)
+            kwargs.setdefault("term_size", term_size)
+
         from pudb.debugger import Debugger
         dbg = Debugger(**kwargs)
 
@@ -39,12 +112,23 @@ def _get_debugger(**kwargs):
     else:
         return CURRENT_DEBUGGER[0]
 
-import signal
+
+def _have_debugger():
+    return len(CURRENT_DEBUGGER) > 0
+
+
+import signal  # noqa
 DEFAULT_SIGNAL = signal.SIGINT
 del signal
 
 
-def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
+def runmodule(*args, **kwargs):
+    kwargs["run_as_module"] = True
+    runscript(*args, **kwargs)
+
+
+def runscript(mainpyfile, args=None, pre_run="", steal_output=False,
+              run_as_module=False):
     dbg = _get_debugger(steal_output=steal_output)
 
     # Note on saving/restoring sys.argv: it's a good idea when sys.argv was
@@ -56,7 +140,10 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
     import sys
     if args is not None:
         prev_sys_argv = sys.argv[:]
-        sys.argv = [mainpyfile] + args
+        if run_as_module:
+            sys.argv = args
+        else:
+            sys.argv = [mainpyfile] + args
 
     # replace pudb's dir with script's dir in front of module search path.
     from os.path import dirname
@@ -74,12 +161,20 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
         status_msg = ""
 
         try:
-            dbg._runscript(mainpyfile)
-        except SystemExit:
-            se = sys.exc_info()[1]
-            status_msg = "The debuggee exited normally with " \
-                    "status code %s.\n\n" % se.code
-        except:
+            if run_as_module:
+                try:
+                    dbg._runmodule(mainpyfile)
+                except ImportError as e:
+                    print(e, file=sys.stderr)
+                    sys.exit(1)
+            else:
+                try:
+                    dbg._runscript(mainpyfile)
+                except SystemExit:
+                    se = sys.exc_info()[1]
+                    status_msg = "The debuggee exited normally with " \
+                            "status code %s.\n\n" % se.code
+        except Exception:
             dbg.post_mortem = True
             dbg.interaction(None, sys.exc_info())
 
@@ -87,7 +182,7 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
             import urwid
             pre_run_edit = urwid.Edit("", pre_run)
 
-            if not CONFIG["prompt_on_quit"]:
+            if not load_config()["prompt_on_quit"]:
                 return
 
             result = dbg.ui.call_with_ui(dbg.ui.dialog,
@@ -145,12 +240,24 @@ def runcall(*args, **kwds):
     return _get_debugger().runcall(*args, **kwds)
 
 
-def set_trace():
+def set_trace(paused=True):
+    """
+    Start the debugger
+
+    If paused=False (the default is True), the debugger will not stop here
+    (same as immediately pressing 'c' to continue).
+    """
     import sys
     dbg = _get_debugger()
 
-    set_interrupt_handler()
-    dbg.set_trace(sys._getframe().f_back)
+    import threading
+    if isinstance(threading.current_thread(), threading._MainThread):
+        set_interrupt_handler()
+
+    dbg.set_trace(sys._getframe().f_back, paused=paused)
+
+
+start = set_trace
 
 
 def _interrupt_handler(signum, frame):
@@ -158,7 +265,7 @@ def _interrupt_handler(signum, frame):
     _get_debugger().set_trace(frame, as_breakpoint=False)
 
 
-def set_interrupt_handler(interrupt_signal=DEFAULT_SIGNAL):
+def set_interrupt_handler(interrupt_signal=None):
     """
     Set up an interrupt handler, to activate PuDB when Python receives the
     signal `interrupt_signal`.  By default it is SIGINT (i.e., Ctrl-c).
@@ -175,7 +282,13 @@ def set_interrupt_handler(interrupt_signal=DEFAULT_SIGNAL):
     >>> pudb.DEFAULT_SIGNAL = signal.SIGALRM
 
     Note, this may not work if you use threads or subprocesses.
+
+    Note, this only works when called from the main thread.
     """
+
+    if interrupt_signal is None:
+        interrupt_signal = DEFAULT_SIGNAL
+
     import signal
     old_handler = signal.getsignal(interrupt_signal)
 
@@ -186,11 +299,18 @@ def set_interrupt_handler(interrupt_signal=DEFAULT_SIGNAL):
         from warnings import warn
         if old_handler is None:
             # This is the documented meaning of getsignal()->None.
-            old_handler = 'not installed from python'
+            old_handler = "not installed from python"
         return warn("A non-default handler for signal %d is already installed (%s). "
                 "Skipping pudb interrupt support."
                 % (interrupt_signal, old_handler))
 
+    import threading
+    if not isinstance(threading.current_thread(), threading._MainThread):
+        from warnings import warn
+        # Setting signals from a non-main thread will not work
+        return warn("Setting the interrupt handler can only be done on the main "
+                "thread. The interrupt handler was NOT installed.")
+
     try:
         signal.signal(interrupt_signal, _interrupt_handler)
     except ValueError:
@@ -224,7 +344,7 @@ def pm():
         e_value = sys.last_value
         tb = sys.last_traceback
     except AttributeError:
-        ## No exception on record. Do nothing.
+        # No exception on record. Do nothing.
         return
     post_mortem(tb, e_type, e_value)
 
