@@ -10,7 +10,7 @@ if PY3:
     text_type = str
 
     def execfile(fname, globs, locs=None):
-        exec(compile(open(fname).read(), fname, 'exec'), globs, locs or globs)
+        exec(compile(open(fname).read(), fname, "exec"), globs, locs or globs)
 else:
     raw_input = raw_input
     xrange = xrange
@@ -18,3 +18,11 @@ else:
     string_types = (basestring,)  # noqa: F821
     text_type = unicode  # noqa: F821
     execfile = execfile
+
+
+try:
+    import builtins
+    from configparser import ConfigParser
+except ImportError:
+    import __builtin__ as builtins  # noqa: F401
+    from ConfigParser import ConfigParser  # noqa: F401
