@@ -1,5 +1,31 @@
 from __future__ import absolute_import, division, print_function
 
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
 import urwid
 
 
@@ -7,7 +33,7 @@ TABSTOP = 8
 
 
 class SourceLine(urwid.FlowWidget):
-    def __init__(self, dbg_ui, text, line_nr='', attr=None, has_breakpoint=False):
+    def __init__(self, dbg_ui, text, line_nr="", attr=None, has_breakpoint=False):
         self.dbg_ui = dbg_ui
         self.text = text
         self.attr = attr
@@ -41,7 +67,7 @@ class SourceLine(urwid.FlowWidget):
         maxcol = size[0]
         hscroll = self.dbg_ui.source_hscroll_start
 
-        # attrs is a list of words like 'focused' and 'breakpoint'
+        # attrs is a list of words like "focused" and "breakpoint"
         attrs = []
 
         if self.is_current:
@@ -101,11 +127,11 @@ class SourceLine(urwid.FlowWidget):
                 # means: gobble up remainder of text and rest of line
                 # and fill with attribute
 
-                l = hscroll+maxcol
+                rowlen = hscroll+maxcol
                 remaining_text = text[i:]
                 encoded_seg_text, seg_cs = apply_target_encoding(
-                        remaining_text + l*" ")
-                encoded_attr.append((seg_attr, len(remaining_text)+l))
+                        remaining_text + rowlen*" ")
+                encoded_attr.append((seg_attr, len(remaining_text)+rowlen))
             else:
                 unencoded_seg_text = text[i:i+seg_len]
                 encoded_seg_text, seg_cs = apply_target_encoding(unencoded_seg_text)
@@ -169,7 +195,7 @@ def format_source(debugger_ui, lines, breakpoints):
                 t.Keyword.Namespace: "namespace",
                 t.Token.Argument: "argument",
                 t.Token.Dunder: "dunder",
-                t.Token.Keyword2: 'keyword2',
+                t.Token.Keyword2: "keyword2",
                 t.Keyword: "keyword",
                 t.Literal: "literal",
                 t.Name.Exception: "exception",
@@ -193,20 +219,20 @@ def format_source(debugger_ui, lines, breakpoints):
         # associated strings to new token types.
         ATTR_TRANSLATE = {  # noqa: N806
                 t.Keyword: {
-                    'class': t.Token.Keyword2,
-                    'def': t.Token.Keyword2,
-                    'exec': t.Token.Keyword2,
-                    'lambda': t.Token.Keyword2,
-                    'print': t.Token.Keyword2,
+                    "class": t.Token.Keyword2,
+                    "def": t.Token.Keyword2,
+                    "exec": t.Token.Keyword2,
+                    "lambda": t.Token.Keyword2,
+                    "print": t.Token.Keyword2,
                     },
                 t.Operator: {
-                    '.': t.Token,
+                    ".": t.Token,
                     },
                 t.Name.Builtin.Pseudo: {
-                    'self': t.Token,
+                    "self": t.Token,
                     },
                 t.Name.Builtin: {
-                    'object': t.Name.Class,
+                    "object": t.Name.Class,
                     },
                 }
 
@@ -236,7 +262,7 @@ def format_source(debugger_ui, lines, breakpoints):
                     # Translate dunder method tokens
                     if ttype == (
                             t.Name.Function
-                            and s.startswith('__') and s.endswith('__')
+                            and s.startswith("__") and s.endswith("__")
                             ):
                         ttype = t.Token.Dunder
 
@@ -277,21 +303,21 @@ def format_source(debugger_ui, lines, breakpoints):
                 if subself.current_line:
                     shipout_line()
 
-        highlight("".join(l.expandtabs(TABSTOP) for l in lines),
+        highlight("".join(line.expandtabs(TABSTOP) for line in lines),
                 PythonLexer(stripnl=False), UrwidFormatter())
 
         return result
 
 
 class ParseState(object):
-    '''States for the ArgumentParser class'''
+    """States for the ArgumentParser class"""
     idle = 1
     found_function = 2
     found_open_paren = 3
 
 
 class ArgumentParser(object):
-    '''Parse source code tokens and identify function arguments.
+    """Parse source code tokens and identify function arguments.
 
     This parser implements a state machine which accepts
     Pygments tokens, delivered sequentially from the beginning
@@ -303,7 +329,7 @@ class ArgumentParser(object):
     argument, it returns the correct token type for that
     item (the caller should then replace the associated item's
     token type with the returned type)
-    '''
+    """
 
     def __init__(self, pygments_token):
         self.t = pygments_token
@@ -311,22 +337,22 @@ class ArgumentParser(object):
         self.paren_level = 0
 
     def parse_token(self, token, s):
-        '''Parse token. Return None or replacement token type'''
+        """Parse token. Return None or replacement token type"""
         if self.state == ParseState.idle:
             if token is self.t.Name.Function:
                 self.state = ParseState.found_function
                 self.paren_level = 0
         elif self.state == ParseState.found_function:
-            if token is self.t.Punctuation and s == '(':
+            if token is self.t.Punctuation and s == "(":
                 self.state = ParseState.found_open_paren
                 self.paren_level = 1
         else:
-            if ((token is self.t.Name) or
-                    (token is self.t.Name.Builtin.Pseudo and s == 'self')):
+            if ((token is self.t.Name)
+                    or (token is self.t.Name.Builtin.Pseudo and s == "self")):
                 return self.t.Token.Argument
-            elif token is self.t.Punctuation and s == ')':
+            elif token is self.t.Punctuation and s == ")":
                 self.paren_level -= 1
-            elif token is self.t.Punctuation and s == '(':
+            elif token is self.t.Punctuation and s == "(":
                 self.paren_level += 1
             if self.paren_level == 0:
                 self.state = ParseState.idle
