@@ -1,5 +1,31 @@
 from __future__ import absolute_import, division, print_function
 
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
 THEMES = [
         "classic",
         "vim",
@@ -980,7 +1006,7 @@ def get_palette(may_use_fancy_formats, theme="classic"):
 
             from os.path import expanduser, expandvars
             execfile(expanduser(expandvars(theme)), symbols)
-        except:
+        except Exception:
             print("Error when importing theme:")
             from traceback import print_exc
             print_exc()
@@ -998,9 +1024,9 @@ def get_palette(may_use_fancy_formats, theme="classic"):
 
         # Convert hNNN syntax to equivalent #RGB value
         # (https://github.com/wardi/urwid/issues/24)
-        if fg_color.startswith('h') or bg_color.startswith('h'):
+        if fg_color.startswith("h") or bg_color.startswith("h"):
             attr = urwid.AttrSpec(fg_color, bg_color, colors=256)
-            palette_list.append((setting_name, 'default', 'default', 'default',
+            palette_list.append((setting_name, "default", "default", "default",
                 attr.foreground,
                 attr.background))
         else:
