@@ -1,9 +1,39 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import absolute_import, division, print_function
+
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
 # {{{ constants and imports
 
-from __future__ import absolute_import, division, print_function
 import urwid
+import inspect
+
+from pudb.lowlevel import ui_log
 
 try:
     import numpy
@@ -11,14 +41,12 @@ try:
 except ImportError:
     HAVE_NUMPY = 0
 
-from pudb.py3compat import PY3, execfile, raw_input, xrange, \
-        integer_types, string_types
-if PY3:
-    ELLIPSIS = '…'
-else:
-    ELLIPSIS = unicode('…', 'utf-8')  # noqa: F821
+from pudb.py3compat import execfile, raw_input, xrange, \
+        integer_types, string_types, text_type
 
-from pudb.debugger import CONFIG
+ELLIPSIS = "…"
+
+from pudb.ui_tools import text_width
 
 # }}}
 
@@ -41,11 +69,14 @@ class FrameVarInfo(object):
 
 class InspectInfo(object):
     def __init__(self):
+        # Do not globalize: cyclic import
+        from pudb.debugger import CONFIG
+
         self.show_detail = False
         self.display_type = CONFIG["stringifier"]
         self.highlighted = False
         self.repeated_at_top = False
-        self.access_level = "public"
+        self.access_level = CONFIG["default_variables_access_level"]
         self.show_methods = False
         self.wrap = CONFIG["wrap_variables"]
 
@@ -83,19 +114,34 @@ STR_SAFE_TYPES = get_str_safe_types()
 # {{{ widget
 
 class VariableWidget(urwid.FlowWidget):
-    def __init__(self, prefix, var_label, value_str, id_path=None, attr_prefix=None,
-            watch_expr=None, iinfo=None):
-        self.prefix = prefix
+    PREFIX = "| "
+
+    def __init__(self, parent, var_label, value_str, id_path=None,
+            attr_prefix=None, watch_expr=None, iinfo=None):
+        self.parent = parent
+        self.nesting_level = 0 if parent is None else parent.nesting_level + 1
+        self.prefix = self.PREFIX * self.nesting_level
         self.var_label = var_label
         self.value_str = value_str
         self.id_path = id_path
         self.attr_prefix = attr_prefix or "var"
         self.watch_expr = watch_expr
         if iinfo is None:
+            # Do not globalize: cyclic import
+            from pudb.debugger import CONFIG
+
             self.wrap = CONFIG["wrap_variables"]
         else:
             self.wrap = iinfo.wrap
 
+    def __str__(self):
+        return ("VariableWidget: {value_str}, level {nesting_level}, at {id_path}"
+                .format(
+                    value_str=self.value_str,
+                    nesting_level=self.nesting_level,
+                    id_path=self.id_path,
+                ))
+
     def selectable(self):
         return True
 
@@ -103,14 +149,14 @@ class VariableWidget(urwid.FlowWidget):
 
     def _get_text(self, size):
         maxcol = size[0] - len(self.prefix)  # self.prefix is a padding
-        var_label = self.var_label or ''
-        value_str = self.value_str or ''
+        var_label = self.var_label or ""
+        value_str = self.value_str or ""
         alltext = var_label + ": " + value_str
         # The first line is not indented
         firstline = self.prefix + alltext[:maxcol]
         if not alltext[maxcol:]:
             return [firstline]
-        fulllines, rest = divmod(len(alltext) - maxcol, maxcol - 2)
+        fulllines, rest = divmod(text_width(alltext) - maxcol, maxcol - 2)
         restlines = [alltext[(maxcol - 2)*i + maxcol:(maxcol - 2)*i + 2*maxcol - 2]
             for i in xrange(fulllines + bool(rest))]
         return [firstline] + ["  " + self.prefix + i for i in restlines]
@@ -121,7 +167,7 @@ class VariableWidget(urwid.FlowWidget):
 
         if (self.value_str is not None
                 and self.var_label is not None
-                and len(self.prefix) + len(self.var_label) > self.SIZE_LIMIT):
+                and len(self.prefix) + text_width(self.var_label) > self.SIZE_LIMIT):
             return 2
         else:
             return 1
@@ -135,20 +181,21 @@ class VariableWidget(urwid.FlowWidget):
         else:
             apfx = self.attr_prefix+" "
 
-        var_label = self.var_label or ''
+        var_label = self.var_label or ""
 
         if self.wrap:
             text = self._get_text(size)
 
-            extralabel_full, extralabel_rem = divmod(len(var_label[maxcol:]), maxcol)
-            totallen = sum([len(i) for i in text])
+            extralabel_full, extralabel_rem = divmod(
+                    text_width(var_label[maxcol:]), maxcol)
+            totallen = sum([text_width(i) for i in text])
             labellen = (
                     len(self.prefix)  # Padding of first line
 
                     + (len(self.prefix) + 2)  # Padding of subsequent lines
                     * (extralabel_full + bool(extralabel_rem))
 
-                    + len(var_label)
+                    + text_width(var_label)
 
                     + 2  # for ": "
                     )
@@ -163,33 +210,37 @@ class VariableWidget(urwid.FlowWidget):
 
             return make_canvas(text, attr, maxcol, apfx+"value")
 
+        lprefix = len(self.prefix)
+
         if self.value_str is not None:
             if self.var_label is not None:
-                if len(self.prefix) + len(self.var_label) > self.SIZE_LIMIT:
+                if len(self.prefix) + text_width(self.var_label) > self.SIZE_LIMIT:
                     # label too long? generate separate value line
                     text = [self.prefix + self.var_label,
                             self.prefix+"  " + self.value_str]
 
-                    attr = [[(apfx+"label", len(self.prefix)+len(self.var_label))],
-                            [(apfx+"value", len(self.prefix)+2+len(self.value_str))]]
+                    attr = [
+                        [(apfx+"label", lprefix+text_width(self.var_label))],
+                        [(apfx+"value", lprefix+2+text_width(self.value_str))]
+                        ]
                 else:
                     text = [self.prefix + self.var_label + ": " + self.value_str]
 
                     attr = [[
-                            (apfx+"label", len(self.prefix)+len(self.var_label)+2),
-                            (apfx+"value", len(self.value_str)),
+                            (apfx+"label", lprefix+text_width(self.var_label)+2),
+                            (apfx+"value", text_width(self.value_str)),
                             ]]
             else:
                 text = [self.prefix + self.value_str]
 
                 attr = [[
                         (apfx+"label", len(self.prefix)),
-                        (apfx+"value", len(self.value_str)),
+                        (apfx+"value", text_width(self.value_str)),
                         ]]
         else:
             text = [self.prefix + self.var_label]
 
-            attr = [[(apfx+"label", len(self.prefix) + len(self.var_label)), ]]
+            attr = [[(apfx+"label", lprefix + text_width(self.var_label)), ]]
 
         # Ellipses to show text was cut off
         #encoding = urwid.util.detected_encoding
@@ -208,7 +259,7 @@ class VariableWidget(urwid.FlowWidget):
                     # + unicode(u'…')) + unicode(text[i][maxcol-2:])
         else:
             for i in xrange(len(text)):
-                if len(text[i]) > maxcol:
+                if text_width(text[i]) > maxcol:
                     text[i] = text[i][:maxcol-3] + "..."
 
         return make_canvas(text, attr, maxcol, apfx+"value")
@@ -224,17 +275,42 @@ custom_stringifier_dict = {}
 
 def type_stringifier(value):
     if HAVE_NUMPY and isinstance(value, numpy.ndarray):
-        return "ndarray %s %s" % (value.dtype, value.shape)
+        return text_type("%s(%s) %s") % (
+                type(value).__name__, value.dtype, value.shape)
+
+    elif HAVE_NUMPY and isinstance(value, numpy.number):
+        return text_type("%s (%s)" % (value, value.dtype))
+
     elif isinstance(value, STR_SAFE_TYPES):
         try:
-            return str(value)
+            return text_type(value)
         except Exception:
-            pass
+            message = "string safe type stringifier failed"
+            ui_log.exception(message)
+            return "!! %s !!" % message
 
-    return type(value).__name__
+    elif hasattr(type(value), "safely_stringify_for_pudb"):
+        try:
+            # (E.g.) Mock objects will pretend to have this
+            # and return nonsense.
+            result = value.safely_stringify_for_pudb()
+        except Exception:
+            message = "safely_stringify_for_pudb call failed"
+            ui_log.exception(message)
+            result = "!! %s !!" % message
+
+        if isinstance(result, string_types):
+            return text_type(result)
+
+    elif type(value) in [set, frozenset, list, tuple, dict]:
+        return text_type("%s (%s)") % (type(value).__name__, len(value))
+
+    return text_type(type(value).__name__)
 
 
 def get_stringifier(iinfo):
+    """Return a function that turns an object into a Unicode text object."""
+
     if iinfo.display_type == "type":
         return type_stringifier
     elif iinfo.display_type == "repr":
@@ -246,44 +322,43 @@ def get_stringifier(iinfo):
             if not custom_stringifier_dict:  # Only execfile once
                 from os.path import expanduser
                 execfile(expanduser(iinfo.display_type), custom_stringifier_dict)
-        except:
+        except Exception:
             print("Error when importing custom stringifier:")
             from traceback import print_exc
             print_exc()
             raw_input("Hit enter:")
-            return lambda value: "ERROR: Invalid custom stringifier file."
+            return lambda value: text_type("ERROR: Invalid custom stringifier file.")
         else:
             if "pudb_stringifier" not in custom_stringifier_dict:
                 print("%s does not contain a function named pudb_stringifier at "
                       "the module level." % iinfo.display_type)
                 raw_input("Hit enter:")
-                return lambda value: ("ERROR: Invalid custom stringifier file: "
-                "pudb_stringifer not defined.")
+                return lambda value: text_type(
+                        "ERROR: Invalid custom stringifier file: "
+                        "pudb_stringifer not defined.")
             else:
                 return (lambda value:
-                    str(custom_stringifier_dict["pudb_stringifier"](value)))
+                    text_type(custom_stringifier_dict["pudb_stringifier"](value)))
 
 
 # {{{ tree walking
 
 class ValueWalker:
-    PREFIX = "| "
-
     def __init__(self, frame_var_info):
         self.frame_var_info = frame_var_info
 
-    def walk_value(self, prefix, label, value, id_path=None, attr_prefix=None):
+    def walk_value(self, parent, label, value, id_path=None, attr_prefix=None):
         if id_path is None:
             id_path = label
 
         iinfo = self.frame_var_info.get_inspect_info(id_path, read_only=True)
 
         if isinstance(value, integer_types + (float, complex)):
-            self.add_item(prefix, label, repr(value), id_path, attr_prefix)
+            self.add_item(parent, label, repr(value), id_path, attr_prefix)
         elif isinstance(value, string_types):
-            self.add_item(prefix, label, repr(value), id_path, attr_prefix)
+            self.add_item(parent, label, repr(value), id_path, attr_prefix)
         elif value is None:
-            self.add_item(prefix, label, repr(value), id_path, attr_prefix)
+            self.add_item(parent, label, repr(value), id_path, attr_prefix)
         else:
             try:
                 displayed_value = get_stringifier(iinfo)(value)
@@ -292,6 +367,7 @@ class ValueWalker:
                 # repr() on a random object.
                 displayed_value = type_stringifier(value) \
                                 + " (!! %s error !!)" % iinfo.display_type
+                ui_log.exception("stringifier failed")
 
             if iinfo.show_detail:
                 if iinfo.access_level == "public":
@@ -304,8 +380,8 @@ class ValueWalker:
                     marker += "+()"
                 displayed_value += " [%s]" % marker
 
-            self.add_item(prefix, label,
-                displayed_value, id_path, attr_prefix)
+            new_parent_item = self.add_item(parent, label, displayed_value,
+                id_path, attr_prefix)
 
             if not iinfo.show_detail:
                 return
@@ -317,41 +393,44 @@ class ValueWalker:
                         cont_id_path = "%s.cont-%d" % (id_path, i)
                         if not self.frame_var_info.get_inspect_info(
                                 cont_id_path, read_only=True).show_detail:
-                            self.add_item(prefix+self.PREFIX, "...",
-                                    None, cont_id_path)
+                            self.add_item(new_parent_item, "...", None,
+                                cont_id_path)
                             break
 
-                    self.walk_value(prefix+self.PREFIX, None, entry,
+                    self.walk_value(new_parent_item, None, entry,
                         "%s[%d]" % (id_path, i))
                 if not value:
-                    self.add_item(prefix+self.PREFIX, "<empty>", None)
+                    self.add_item(new_parent_item, "<empty>", None)
                 return
 
             # containers --------------------------------------------------
             key_it = None
 
             try:
-                if PY3:
-                    key_it = value.keys()
-                else:
-                    key_it = value.iterkeys()
-            except:
+                key_it = value.keys()
+            except AttributeError:
+                # keys or iterkeys doesn't exist, not worth mentioning!
                 pass
+            except Exception:
+                ui_log.exception("Failed to obtain key iterator")
 
             if key_it is None:
                 try:
-                    l = len(value)
-                except:
+                    len_value = len(value)
+                except TypeError:
+                    # no __len__ defined on the value, not worth mentioning!
                     pass
+                except Exception:
+                    ui_log.exception("Failed to determine container length")
                 else:
                     try:
                         value[0]
                     except IndexError:
                         key_it = []
-                    except:
-                        pass
+                    except Exception:
+                        ui_log.exception("Item is not iterable")
                     else:
-                        key_it = xrange(l)
+                        key_it = xrange(len_value)
 
             if key_it is not None:
                 cnt = 0
@@ -361,14 +440,14 @@ class ValueWalker:
                         if not self.frame_var_info.get_inspect_info(
                                 cont_id_path, read_only=True).show_detail:
                             self.add_item(
-                                prefix+self.PREFIX, "...", None, cont_id_path)
+                                new_parent_item, "...", None, cont_id_path)
                             break
 
-                    self.walk_value(prefix+self.PREFIX, repr(key), value[key],
+                    self.walk_value(new_parent_item, repr(key), value[key],
                         "%s[%r]" % (id_path, key))
                     cnt += 1
                 if not cnt:
-                    self.add_item(prefix+self.PREFIX, "<empty>", None)
+                    self.add_item(new_parent_item, "<empty>", None)
                 return
 
             # class types -------------------------------------------------
@@ -376,8 +455,8 @@ class ValueWalker:
 
             try:
                 key_its.append(dir(value))
-            except:
-                pass
+            except Exception:
+                ui_log.exception("Failed to look up attributes")
 
             keys = [key
                     for ki in key_its
@@ -398,13 +477,13 @@ class ValueWalker:
 
                 try:
                     attr_value = getattr(value, key)
-                    if callable(attr_value) and not iinfo.show_methods:
+                    if inspect.isroutine(attr_value) and not iinfo.show_methods:
                         cnt_omitted_methods += 1
                         continue
-                except:
+                except Exception:
                     attr_value = WatchEvalError()
 
-                self.walk_value(prefix+self.PREFIX,
+                self.walk_value(new_parent_item,
                         ".%s" % key, attr_value,
                         "%s.%s" % (id_path, key))
 
@@ -415,10 +494,10 @@ class ValueWalker:
                     label = "<omitted methods>"
                 else:
                     label = "<empty>"
-                self.add_item(prefix+self.PREFIX, label, None)
+                self.add_item(new_parent_item, label, None)
 
             if not key_its:
-                self.add_item(prefix+self.PREFIX, "<?>", None)
+                self.add_item(new_parent_item, "<?>", None)
 
 
 class BasicValueWalker(ValueWalker):
@@ -427,13 +506,15 @@ class BasicValueWalker(ValueWalker):
 
         self.widget_list = []
 
-    def add_item(self, prefix, var_label, value_str, id_path=None, attr_prefix=None):
+    def add_item(self, parent, var_label, value_str, id_path=None, attr_prefix=None):
         iinfo = self.frame_var_info.get_inspect_info(id_path, read_only=True)
         if iinfo.highlighted:
             attr_prefix = "highlighted var"
 
-        self.widget_list.append(VariableWidget(prefix, var_label, value_str,
-            id_path, attr_prefix, iinfo=iinfo))
+        new_item = VariableWidget(parent, var_label, value_str, id_path,
+            attr_prefix, iinfo=iinfo)
+        self.widget_list.append(new_item)
+        return new_item
 
 
 class WatchValueWalker(ValueWalker):
@@ -442,14 +523,15 @@ class WatchValueWalker(ValueWalker):
         self.widget_list = widget_list
         self.watch_expr = watch_expr
 
-    def add_item(self, prefix, var_label, value_str, id_path=None, attr_prefix=None):
+    def add_item(self, parent, var_label, value_str, id_path=None, attr_prefix=None):
         iinfo = self.frame_var_info.get_inspect_info(id_path, read_only=True)
         if iinfo.highlighted:
             attr_prefix = "highlighted var"
 
-        self.widget_list.append(
-                VariableWidget(prefix, var_label, value_str, id_path, attr_prefix,
-                    watch_expr=self.watch_expr, iinfo=iinfo))
+        new_item = VariableWidget(parent, var_label, value_str, id_path,
+            attr_prefix, watch_expr=self.watch_expr, iinfo=iinfo)
+        self.widget_list.append(new_item)
+        return new_item
 
 
 class TopAndMainVariableWalker(ValueWalker):
@@ -461,7 +543,7 @@ class TopAndMainVariableWalker(ValueWalker):
 
         self.top_id_path_prefixes = []
 
-    def add_item(self, prefix, var_label, value_str, id_path=None, attr_prefix=None):
+    def add_item(self, parent, var_label, value_str, id_path=None, attr_prefix=None):
         iinfo = self.frame_var_info.get_inspect_info(id_path, read_only=True)
         if iinfo.highlighted:
             attr_prefix = "highlighted var"
@@ -475,11 +557,13 @@ class TopAndMainVariableWalker(ValueWalker):
                 repeated_at_top = True
 
         if repeated_at_top:
-            self.top_widget_list.append(VariableWidget(prefix, var_label,
+            self.top_widget_list.append(VariableWidget(parent, var_label,
                 value_str, id_path, attr_prefix, iinfo=iinfo))
 
-        self.main_widget_list.append(VariableWidget(prefix, var_label,
-            value_str, id_path, attr_prefix, iinfo=iinfo))
+        new_item = VariableWidget(parent, var_label, value_str, id_path,
+            attr_prefix, iinfo=iinfo)
+        self.main_widget_list.append(new_item)
+        return new_item
 
 # }}}
 
@@ -491,7 +575,7 @@ SEPARATOR = urwid.AttrMap(urwid.Text(""), "variable separator")
 
 def make_var_view(frame_var_info, locals, globals):
     vars = list(locals.keys())
-    vars.sort(key=lambda n: n.lower())
+    vars.sort(key=str.lower)
 
     tmv_walker = TopAndMainVariableWalker(frame_var_info)
     ret_walker = BasicValueWalker(frame_var_info)
@@ -500,19 +584,19 @@ def make_var_view(frame_var_info, locals, globals):
     for watch_expr in frame_var_info.watches:
         try:
             value = eval(watch_expr.expression, globals, locals)
-        except:
+        except Exception:
             value = WatchEvalError()
 
         WatchValueWalker(frame_var_info, watch_widget_list, watch_expr) \
-                .walk_value("", watch_expr.expression, value)
+                .walk_value(None, watch_expr.expression, value)
 
     if "__return__" in vars:
-        ret_walker.walk_value("", "Return", locals["__return__"],
+        ret_walker.walk_value(None, "Return", locals["__return__"],
                 attr_prefix="return")
 
     for var in vars:
-        if not var[0] in "_.":
-            tmv_walker.walk_value("", var, locals[var])
+        if not (var.startswith("__") and var.endswith("__")):
+            tmv_walker.walk_value(None, var, locals[var])
 
     result = tmv_walker.main_widget_list
 
